@@ -1,4 +1,4 @@
-/* $Id: client.c,v 1.25 2007/12/06 09:46:21 nicm Exp $ */
+/* $Id: client.c,v 1.29 2008/06/07 07:27:28 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -44,15 +44,16 @@ client_init(const char *path, struct client_ctx *cctx, int start_server)
 	size_t				size;
 	int				mode;
 	u_int				retries;
+	struct buffer		       *b;
+	pid_t				pid;
 
+	pid = 0;
 	retries = 0;
 retry:
 	if (stat(path, &sb) != 0) {
-		if (start_server && errno == ENOENT && retries < 10) {
+		if (start_server && errno == ENOENT) {
 			if (server_start(path) != 0)
-				return (-1);
-			usleep(10000);
-			retries++;
+				goto no_start;
 			goto retry;
 		}
 		goto fail;
@@ -78,9 +79,8 @@ retry:
 		if (start_server && errno == ECONNREFUSED && retries < 10) {
 			if (unlink(path) != 0)
 				goto fail;
-			usleep(10000);
-			retries++;
-			goto retry;
+			if (server_start(path) != 0)
+				goto no_start;
 		}
 		goto fail;
 	}
@@ -99,15 +99,23 @@ retry:
 		data.sy = ws.ws_row;
 		if (ttyname_r(STDIN_FILENO, data.tty, sizeof data.tty) != 0)
 			fatal("ttyname_r failed");
-		client_write_server(cctx, MSG_IDENTIFY, &data, sizeof data);
-		cmd_send_string(cctx->srv_out, getenv("TERM"));
+
+		b = buffer_create(BUFSIZ);
+		cmd_send_string(b, getenv("TERM"));
+		client_write_server2(cctx, MSG_IDENTIFY,
+		    &data, sizeof data, BUFFER_OUT(b), BUFFER_USED(b));
+		buffer_destroy(b);
 	}
 
 	return (0);
 
+no_start:
+	log_warnx("server failed to start");
+	return (1);
+
 fail:
 	log_warn("server not found");
-	return (-1);
+	return (1);
 }
 
 int
@@ -130,6 +138,19 @@ client_main(struct client_ctx *cctx)
 		if (sigwinch)
 			client_handle_winch(cctx);
 
+		switch (client_msg_dispatch(cctx, &error)) {
+		case -1:
+			goto out;
+		case 0:
+			/* May be more in buffer, don't let poll block. */
+			timeout = 0;
+			break;
+		default:
+			/* Out of data, poll may block. */
+			timeout = INFTIM;
+			break;
+		}
+
 		pfd.fd = cctx->srv_fd;
 		pfd.events = POLLIN;
 		if (BUFFER_USED(cctx->srv_out) > 0)
@@ -143,19 +164,6 @@ client_main(struct client_ctx *cctx)
 
 		if (buffer_poll(&pfd, cctx->srv_in, cctx->srv_out) != 0)
 			goto server_dead;
-
-		switch (client_msg_dispatch(cctx, &error)) {
-		case -1:
-			goto out;
-		case 0:
-			/* May be more in buffer, don't let poll block. */
-			timeout = 0;
-			break;
-		default:
-			/* Out of data, poll may block. */
-			timeout = INFTIM;
-			break;
-		}
 	}
 
 out:
