@@ -1,4 +1,4 @@
-/* $Id: cmd-copy-mode.c,v 1.4 2007/12/06 10:04:42 nicm Exp $ */
+/* $Id: cmd-copy-mode.c,v 1.10 2008/06/05 21:25:00 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,34 +18,37 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-#include <stdlib.h>
-
 #include "tmux.h"
 
 /*
  * Enter copy mode. Only valid when bound to a key.
  */
 
-void	cmd_copy_mode_exec(void *, struct cmd_ctx *);
+void	cmd_copy_mode_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_copy_mode_entry = {
-	"copy-mode", NULL, "",
-	CMD_NOCLIENT,
-	NULL,
+	"copy-mode", NULL, 
+	CMD_TARGET_WINDOW_USAGE,
+	0,
+	cmd_target_init,
+	cmd_target_parse,
 	cmd_copy_mode_exec,
-	NULL,
-	NULL,
+	cmd_target_send,
+	cmd_target_recv,
+	cmd_target_free,
 	NULL
 };
 
 void
-cmd_copy_mode_exec(unused void *ptr, struct cmd_ctx *ctx)
+cmd_copy_mode_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct window	*w = ctx->session->curw->window;
+	struct cmd_target_data	*data = self->data;
+	struct winlink		*wl;
 
-	if (ctx->flags & CMD_KEY)
-		window_set_mode(w, &window_copy_mode);
+	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
+		return;
+
+	window_set_mode(wl->window, &window_copy_mode);
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
