@@ -1,4 +1,4 @@
-/* $Id: cmd-next-window.c,v 1.6 2007/12/06 09:46:22 nicm Exp $ */
+/* $Id: cmd-next-window.c,v 1.12 2008/06/06 20:02:27 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,34 +18,41 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-#include <string.h>
-
 #include "tmux.h"
 
 /*
  * Move to next window.
  */
 
-void	cmd_next_window_exec(void *, struct cmd_ctx *);
+void	cmd_next_window_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_next_window_entry = {
-	"next-window", "next", "",
-	CMD_NOCLIENT,
-	NULL,
+	"next-window", "next",
+	CMD_TARGET_SESSION_USAGE,
+	0,
+	cmd_target_init,
+	cmd_target_parse,
 	cmd_next_window_exec,
-	NULL,
-	NULL,
-	NULL
+	cmd_target_send,
+	cmd_target_recv,
+	cmd_target_free,
+	cmd_target_print
 };
 
 void
-cmd_next_window_exec(unused void *ptr, struct cmd_ctx *ctx)
+cmd_next_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	if (session_next(ctx->session) == 0)
-		server_redraw_session(ctx->session);
+	struct cmd_target_data	*data = self->data;
+	struct session		*s;
+
+	if ((s = cmd_find_session(ctx, data->target)) == NULL)
+		return;
+
+	if (session_next(s) == 0)
+		server_redraw_session(s);
 	else
 		ctx->error(ctx, "no next window");
+	recalculate_sizes();
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
