@@ -1,4 +1,4 @@
-/* $Id: cmd-paste-buffer.c,v 1.2 2007/12/06 09:46:22 nicm Exp $ */
+/* $Id: cmd-paste-buffer.c,v 1.8 2008/06/05 21:25:00 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,8 +18,6 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "tmux.h"
@@ -28,28 +26,33 @@
  * Paste paste buffer if present.
  */
 
-void	cmd_paste_buffer_exec(void *, struct cmd_ctx *);
+void	cmd_paste_buffer_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_paste_buffer_entry = {
-	"paste-buffer", NULL, "paste",
-	CMD_NOCLIENT,
-	NULL,
+	"paste-buffer", "paste",
+	CMD_TARGET_WINDOW_USAGE,
+	0,
+	cmd_target_init,
+	cmd_target_parse,
 	cmd_paste_buffer_exec,
-	NULL,
-	NULL,
-	NULL
+	cmd_target_send,
+	cmd_target_recv,
+	cmd_target_free,
+	cmd_target_print
 };
 
 void
-cmd_paste_buffer_exec(unused void *ptr, struct cmd_ctx *ctx)
+cmd_paste_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct window	*w = ctx->session->curw->window;
+	struct cmd_target_data	*data = self->data;
+	struct winlink		*wl;
 
-	if (ctx->flags & CMD_KEY) {
-		if (paste_buffer != NULL && *paste_buffer != '\0') {
-			buffer_write(
-			    w->out, paste_buffer, strlen(paste_buffer));
-		}
+	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
+		return;
+
+	if (paste_buffer != NULL && *paste_buffer != '\0') {
+		buffer_write(
+		    wl->window->out, paste_buffer, strlen(paste_buffer));
 	}
 
 	if (ctx->cmdclient != NULL)
