@@ -1,4 +1,4 @@
-/* $Id: cmd-refresh-client.c,v 1.1 2007/11/16 21:12:31 nicm Exp $ */
+/* $Id: cmd-refresh-client.c,v 1.6 2008/06/05 21:25:00 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,30 +18,37 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-
 #include "tmux.h"
 
 /*
  * Refresh client.
  */
 
-void	cmd_refresh_client_exec(void *, struct cmd_ctx *);
+void	cmd_refresh_client_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_refresh_client_entry = {
-	"refresh-client", "refresh", "",
+	"refresh-client", "refresh",
+	CMD_TARGET_CLIENT_USAGE,
 	0,
-	NULL,
+	cmd_target_init,
+	cmd_target_parse,
 	cmd_refresh_client_exec,
-	NULL,
-	NULL,
-	NULL
+	cmd_target_send,
+	cmd_target_recv,
+	cmd_target_free,
+	cmd_target_print
 };
 
 void
-cmd_refresh_client_exec(unused void *ptr, struct cmd_ctx *ctx)
+cmd_refresh_client_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	server_redraw_client(ctx->client);
+	struct cmd_target_data	*data = self->data;
+	struct client		*c;
+
+	if ((c = cmd_find_client(ctx, data->target)) == NULL)
+		return;
+	
+	server_redraw_client(c);
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
