@@ -1,4 +1,4 @@
-/* $Id: cmd-send-prefix.c,v 1.7 2007/12/06 09:46:22 nicm Exp $ */
+/* $Id: cmd-send-prefix.c,v 1.15 2008/06/15 08:01:54 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,31 +18,38 @@
 
 #include <sys/types.h>
 
-#include <getopt.h>
-#include <unistd.h>
-
 #include "tmux.h"
 
 /*
  * Send prefix key as a key.
  */
 
-void	cmd_send_prefix_exec(void *, struct cmd_ctx *);
+void	cmd_send_prefix_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_send_prefix_entry = {
-	"send-prefix", NULL, NULL,
+	"send-prefix", NULL,
+	CMD_TARGET_WINDOW_USAGE,
 	0,
-	NULL,
+	cmd_target_init,
+	cmd_target_parse,
 	cmd_send_prefix_exec,
-	NULL,
-	NULL,
-	NULL
+	cmd_target_send,
+	cmd_target_recv,
+	cmd_target_free,
+	cmd_target_print
 };
 
 void
-cmd_send_prefix_exec(unused void *ptr, struct cmd_ctx *ctx)
+cmd_send_prefix_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	window_key(ctx->client->session->curw->window, prefix_key);
+	struct cmd_target_data	*data = self->data;
+	struct session		*s;
+	struct winlink		*wl;
+
+	if ((wl = cmd_find_window(ctx, data->target, &s)) == NULL)
+		return;
+
+	window_key(wl->window, options_get_key(&s->options, "prefix-key"));
 
 	if (ctx->cmdclient != NULL)
 		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
