@@ -1,4 +1,4 @@
-/* $Id: cmd-set-option.c,v 1.15 2007/12/06 09:46:22 nicm Exp $ */
+/* $Id: cmd-set-option.c,v 1.28 2008/06/16 06:33:50 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,39 +28,52 @@
  * Set an option.
  */
 
-int	cmd_set_option_parse(void **, int, char **, char **);
-void	cmd_set_option_exec(void *, struct cmd_ctx *);
-void	cmd_set_option_send(void *, struct buffer *);
-void	cmd_set_option_recv(void **, struct buffer *);
-void	cmd_set_option_free(void *);
+int	cmd_set_option_parse(struct cmd *, int, char **, char **);
+void	cmd_set_option_exec(struct cmd *, struct cmd_ctx *);
+void	cmd_set_option_send(struct cmd *, struct buffer *);
+void	cmd_set_option_recv(struct cmd *, struct buffer *);
+void	cmd_set_option_free(struct cmd *);
+void	cmd_set_option_print(struct cmd *, char *, size_t);
 
 struct cmd_set_option_data {
+	char	*target;
+	int	 flag_global;
 	char	*option;
 	char	*value;
 };
 
 const struct cmd_entry cmd_set_option_entry = {
-	"set-option", "set", "option value",
-	CMD_NOCLIENT|CMD_NOSESSION,
+	"set-option", "set",
+	"[-t target-session] option value",
+	0,
+	NULL,
 	cmd_set_option_parse,
 	cmd_set_option_exec,
 	cmd_set_option_send,
 	cmd_set_option_recv,
-	cmd_set_option_free
+	cmd_set_option_free,
+	cmd_set_option_print
 };
 
 int
-cmd_set_option_parse(void **ptr, int argc, char **argv, char **cause)
+cmd_set_option_parse(struct cmd *self, int argc, char **argv, char **cause)
 {
 	struct cmd_set_option_data	*data;
 	int				 opt;
 
-	*ptr = data = xmalloc(sizeof *data);
+	self->data = data = xmalloc(sizeof *data);
+	data->target = NULL;
+	data->flag_global = 1;
 	data->option = NULL;
 	data->value = NULL;
 
-	while ((opt = getopt(argc, argv, "")) != EOF) {
+	while ((opt = getopt(argc, argv, "t:s:")) != EOF) {
 		switch (opt) {
+		case 't':
+			if (data->target == NULL)
+				data->target = xstrdup(optarg);
+			data->flag_global = 0;
+			break;
 		default:
 			goto usage;
 		}
@@ -77,25 +90,33 @@ cmd_set_option_parse(void **ptr, int argc, char **argv, char **cause)
 	return (0);
 
 usage:
-	usage(cause, "%s %s",
-	    cmd_set_option_entry.name, cmd_set_option_entry.usage);
+	xasprintf(cause, "usage: %s %s", self->entry->name, self->entry->usage);
 
-	cmd_set_option_free(data);
+	self->entry->free(self);
 	return (-1);
 }
 
 void
-cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
+cmd_set_option_exec(struct cmd *self, unused struct cmd_ctx *ctx)
 {
-	struct cmd_set_option_data	*data = ptr;
+	struct cmd_set_option_data	*data = self->data;
 	struct client			*c;
+	struct session			*s;
+	struct options			*oo;
 	const char			*errstr;
 	u_int				 i;
 	int				 number, bool, key;
+	u_char				 colour;
 
 	if (data == NULL)
 		return;
 
+	if (data->flag_global ||
+	    ((s = cmd_find_session(ctx, data->target))) == NULL)
+		oo = &global_options;
+	else
+		oo = &s->options;
+
 	if (*data->option == '\0') {
 		ctx->error(ctx, "invalid option");
 		return;
@@ -104,6 +125,8 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 	number = -1;
 	if (data->value != NULL) {
 		number = strtonum(data->value, 0, INT_MAX, &errstr);
+		if (errstr != NULL)
+			number = 0;
 
 		bool = -1;
 		if (number == 1 || strcasecmp(data->value, "on") == 0 ||
@@ -113,7 +136,7 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 		    strcasecmp(data->value, "no") == 0)
 			bool = 0;
 	} else
-		bool = 1;
+		bool = -2;
 
 	if (strcmp(data->option, "prefix") == 0) {
 		if (data->value == NULL) {
@@ -125,13 +148,15 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 			ctx->error(ctx, "unknown key: %s", data->value);
 			return;
 		}
-		prefix_key = key;
+		options_set_key(oo, "prefix-key", key);
 	} else if (strcmp(data->option, "status") == 0) {
 		if (bool == -1) {
 			ctx->error(ctx, "bad value: %s", data->value);
 			return;
 		}
-		status_lines = bool;
+		if (bool == -2)
+			bool = !options_get_number(oo, "status-lines");
+		options_set_number(oo, "status-lines", bool);
 		recalculate_sizes();
 	} else if (strcmp(data->option, "status-fg") == 0) {
 		if (data->value == NULL) {
@@ -143,14 +168,16 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 			ctx->error(ctx, "bad colour: %s", data->value);
 			return;
 		}
-		status_colour &= 0x0f;
-		status_colour |= number << 4;
-		if (status_lines > 0) {
-			for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-				c = ARRAY_ITEM(&clients, i);
-				if (c != NULL && c->session != NULL)
-					server_redraw_client(c);
-			}
+
+		colour = options_get_colours(oo, "status-colour");
+		colour &= 0x0f;
+		colour |= number << 4;
+		options_set_colours(oo, "status-colour", colour);
+
+		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+			c = ARRAY_ITEM(&clients, i);
+			if (c != NULL && c->session != NULL)
+				server_redraw_client(c);
 		}
 	} else if (strcmp(data->option, "status-bg") == 0) {
 		if (data->value == NULL) {
@@ -162,14 +189,16 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 			ctx->error(ctx, "bad colour: %s", data->value);
 			return;
 		}
-		status_colour &= 0xf0;
-		status_colour |= number;
-		if (status_lines > 0) {
-			for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-				c = ARRAY_ITEM(&clients, i);
-				if (c != NULL && c->session != NULL)
-					server_redraw_client(c);
-			}
+
+		colour = options_get_colours(oo, "status-colour");
+		colour &= 0xf0;
+		colour |= number;
+		options_set_colours(oo, "status-colour", colour);
+
+		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+			c = ARRAY_ITEM(&clients, i);
+			if (c != NULL && c->session != NULL)
+				server_redraw_client(c);
 		}
 	} else if (strcmp(data->option, "bell-action") == 0) {
 		if (data->value == NULL) {
@@ -177,32 +206,72 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 			return;
 		}
 		if (strcmp(data->value, "any") == 0)
-			bell_action = BELL_ANY;
+			number = BELL_ANY;
 		else if (strcmp(data->value, "none") == 0)
-			bell_action = BELL_NONE;
+			number = BELL_NONE;
 		else if (strcmp(data->value, "current") == 0)
-			bell_action = BELL_CURRENT;
+			number = BELL_CURRENT;
 		else {
 			ctx->error(ctx, "unknown bell-action: %s", data->value);
 			return;
 		}
+		options_set_number(oo, "bell-action", number);
 	} else if (strcmp(data->option, "default-command") == 0) {
 		if (data->value == NULL) {
 			ctx->error(ctx, "invalid value");
 			return;
 		}
-		xfree(default_command);
-		default_command = xstrdup(data->value);
+		options_set_string(oo, "default-command", "%s", data->value);
 	} else if (strcmp(data->option, "history-limit") == 0) {
-		if (data->value == NULL) {
+		if (data->value == NULL || number == -1) {
 			ctx->error(ctx, "invalid value");
 			return;
 		}
+		if (errstr != NULL) {
+			ctx->error(ctx, "history-limit %s", errstr);
+			return;
+		}
 		if (number > SHRT_MAX) {
 			ctx->error(ctx, "history-limit too big: %u", number);
 			return;
 		}
-		history_limit = number;
+		options_set_number(oo, "history-limit", number);
+	} else if (strcmp(data->option, "status-left") == 0) {
+		if (data->value == NULL) {
+			ctx->error(ctx, "invalid value");
+			return;
+		}
+
+		options_set_string(oo, "status-left", "%s", data->value);
+
+		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+			c = ARRAY_ITEM(&clients, i);
+			if (c != NULL && c->session != NULL)
+				server_redraw_client(c);
+		}
+	} else if (strcmp(data->option, "status-right") == 0) {
+		if (data->value == NULL) {
+			ctx->error(ctx, "invalid value");
+			return;
+		}
+
+		options_set_string(oo, "status-right", "%s", data->value);
+
+		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+			c = ARRAY_ITEM(&clients, i);
+			if (c != NULL && c->session != NULL)
+				server_redraw_client(c);
+		}
+	} else if (strcmp(data->option, "status-interval") == 0) {
+		if (data->value == NULL || number == -1) {
+			ctx->error(ctx, "invalid value");
+			return;
+		}
+		if (errstr != NULL) {
+			ctx->error(ctx, "status-interval %s", errstr);
+			return;
+		}
+		options_set_number(oo, "status-interval", number);
 	} else {
 		ctx->error(ctx, "unknown option: %s", data->option);
 		return;
@@ -213,32 +282,53 @@ cmd_set_option_exec(void *ptr, unused struct cmd_ctx *ctx)
 }
 
 void
-cmd_set_option_send(void *ptr, struct buffer *b)
+cmd_set_option_send(struct cmd *self, struct buffer *b)
 {
-	struct cmd_set_option_data	*data = ptr;
+	struct cmd_set_option_data	*data = self->data;
 
+	buffer_write(b, data, sizeof *data);
+	cmd_send_string(b, data->target);
 	cmd_send_string(b, data->option);
 	cmd_send_string(b, data->value);
 }
 
 void
-cmd_set_option_recv(void **ptr, struct buffer *b)
+cmd_set_option_recv(struct cmd *self, struct buffer *b)
 {
 	struct cmd_set_option_data	*data;
 
-	*ptr = data = xmalloc(sizeof *data);
+	self->data = data = xmalloc(sizeof *data);
+	buffer_read(b, data, sizeof *data);
+	data->target = cmd_recv_string(b);
 	data->option = cmd_recv_string(b);
 	data->value = cmd_recv_string(b);
 }
 
 void
-cmd_set_option_free(void *ptr)
+cmd_set_option_free(struct cmd *self)
 {
-	struct cmd_set_option_data	*data = ptr;
+	struct cmd_set_option_data	*data = self->data;
 
+	if (data->target != NULL)
+		xfree(data->target);
 	if (data->option != NULL)
 		xfree(data->option);
 	if (data->value != NULL)
 		xfree(data->value);
 	xfree(data);
 }
+
+void
+cmd_set_option_print(struct cmd *self, char *buf, size_t len)
+{
+	struct cmd_set_option_data	*data = self->data;
+	size_t				 off = 0;
+
+	off += xsnprintf(buf, len, "%s", self->entry->name);
+	if (data == NULL)
+		return;
+	if (off < len && data->option != NULL)
+		off += xsnprintf(buf + off, len - off, " %s", data->option);
+	if (off < len && data->value != NULL)
+		off += xsnprintf(buf + off, len - off, " %s", data->value);	
+}
