@@ -1,4 +1,4 @@
-/* $Id: cmd.c,v 1.33 2007/12/06 09:46:22 nicm Exp $ */
+/* $Id: cmd.c,v 1.48 2008/06/16 22:03:27 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,9 +17,12 @@
  */
 
 #include <sys/types.h>
+#include <sys/time.h>
 
 #include <getopt.h>
+#include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "tmux.h"
 
@@ -29,6 +32,7 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_copy_mode_entry,
 	&cmd_detach_client_entry,
 	&cmd_has_session_entry,
+	&cmd_kill_server_entry,
 	&cmd_kill_session_entry,
 	&cmd_kill_window_entry,
 	&cmd_last_window_entry,
@@ -47,8 +51,13 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_rename_window_entry,
 	&cmd_scroll_mode_entry,
 	&cmd_select_window_entry,
+	&cmd_send_keys_entry,
 	&cmd_send_prefix_entry,
 	&cmd_set_option_entry,
+	&cmd_set_window_option_entry,
+	&cmd_show_options_entry,
+	&cmd_show_window_options_entry,
+	&cmd_start_server_entry,
 	&cmd_swap_window_entry,
 	&cmd_switch_client_entry,
 	&cmd_unbind_key_entry,
@@ -103,8 +112,9 @@ cmd_parse(int argc, char **argv, char **cause)
 
 	cmd = xmalloc(sizeof *cmd);
 	cmd->entry = entry;
+	cmd->data = NULL;
 	if (entry->parse != NULL) {
-		if (entry->parse(&cmd->data, argc, argv, cause) != 0) {
+		if (entry->parse(cmd, argc, argv, cause) != 0) {
 			xfree(cmd);
 			return (NULL);
 		}
@@ -126,14 +136,14 @@ ambiguous:
 	return (NULL);
 
 usage:
-	usage(cause, "%s %s", entry->name, entry->usage);
+	xasprintf(cause, "usage: %s %s", entry->name, entry->usage);
 	return (NULL);
 }
 
 void
 cmd_exec(struct cmd *cmd, struct cmd_ctx *ctx)
 {
-	cmd->entry->exec(cmd->data, ctx);
+	cmd->entry->exec(cmd, ctx);
 }
 
 void
@@ -154,7 +164,7 @@ cmd_send(struct cmd *cmd, struct buffer *b)
 	buffer_write(b, &n, sizeof n);
 
 	if (cmd->entry->send != NULL)
-		cmd->entry->send(cmd->data, b);
+		cmd->entry->send(cmd, b);
 }
 
 struct cmd *
@@ -179,7 +189,7 @@ cmd_recv(struct buffer *b)
 	cmd->entry = *entryp;
 
 	if (cmd->entry->recv != NULL)
-		cmd->entry->recv(&cmd->data, b);
+		cmd->entry->recv(cmd, b);
 	return (cmd);
 }
 
@@ -187,7 +197,7 @@ void
 cmd_free(struct cmd *cmd)
 {
 	if (cmd->data != NULL && cmd->entry->free != NULL)
-		cmd->entry->free(cmd->data);
+		cmd->entry->free(cmd);
 	xfree(cmd);
 }
 
@@ -225,3 +235,105 @@ cmd_recv_string(struct buffer *b)
 
 	return (s);
 }
+
+struct session *
+cmd_current_session(struct cmd_ctx *ctx)
+{
+	struct msg_command_data	*data = ctx->msgdata;
+	struct timespec		*ts;
+	struct session		*s, *newest = NULL;
+	u_int			 i;
+
+	if (ctx->cursession != NULL)
+		return (ctx->cursession);
+
+	if (data != NULL && data->pid != -1) {
+		if (data->pid != getpid()) {
+			ctx->error(ctx, "wrong server: %lld", data->pid);
+			return (NULL);
+		}
+		if (data->idx > ARRAY_LENGTH(&sessions)) {
+			ctx->error(ctx, "index out of range: %d", data->idx);
+			return (NULL);
+		}
+		if ((s = ARRAY_ITEM(&sessions, data->idx)) == NULL) {
+			ctx->error(ctx, "session doesn't exist: %u", data->idx);
+			return (NULL);
+		}
+		return (s);
+	}
+	
+	ts = NULL;
+	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
+		s = ARRAY_ITEM(&sessions, i); 
+		if (s != NULL && (ts == NULL || timespeccmp(&s->ts, ts, >))) {
+			newest = ARRAY_ITEM(&sessions, i);
+			ts = &s->ts;
+		}
+	}
+	return (newest);
+}
+
+struct client *
+cmd_find_client(struct cmd_ctx *ctx, const char *arg)
+{
+	struct client	*c;
+	
+	if ((c = arg_parse_client(arg)) == NULL)
+		c = ctx->curclient;
+	if (c == NULL) {
+		if (arg != NULL)
+			ctx->error(ctx, "client not found: %s", arg);
+		else
+			ctx->error(ctx, "no client found");
+	}
+	return (c);
+}
+
+struct session *
+cmd_find_session(struct cmd_ctx *ctx, const char *arg)
+{
+	struct session	*s;
+
+	if ((s = arg_parse_session(arg)) == NULL)
+		s = ctx->cursession;
+	if (s == NULL)
+		s = cmd_current_session(ctx);
+	if (s == NULL) {
+		if (arg != NULL)
+			ctx->error(ctx, "session not found: %s", arg);
+		else
+			ctx->error(ctx, "no session found");
+	}
+	return (s);
+}
+
+struct winlink *
+cmd_find_window(struct cmd_ctx *ctx, const char *arg, struct session **sp)
+{
+	struct session	*s;
+	struct winlink	*wl;
+	int		 idx;
+
+	wl = NULL;
+	if (arg_parse_window(arg, &s, &idx) != 0) {
+		ctx->error(ctx, "bad window: %s", arg);
+		return (NULL);
+	}
+	if (s == NULL)
+		s = ctx->cursession;
+	if (s == NULL)
+		s = cmd_current_session(ctx);
+	if (s == NULL)
+		return (NULL);
+	if (sp != NULL)
+		*sp = s;
+
+	if (idx == -1)
+		wl = s->curw;
+	else
+		wl = winlink_find_by_index(&s->windows, idx);
+	if (wl == NULL)
+		ctx->error(ctx, "window not found: %s:%d", s->name, idx);
+	return (wl);
+}
