@@ -1,4 +1,4 @@
-/* $Id: resize.c,v 1.9 2007/12/06 09:46:22 nicm Exp $ */
+/* $Id: resize.c,v 1.15 2008/06/14 16:47:20 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -39,8 +39,7 @@
  *
  * As a side effect, this function updates the SESSION_UNATTACHED flag. This
  * flag is necessary to make sure unattached sessions do not limit the size of
- * windows that are attached both to them and to other sessions which are
- * attached.
+ * windows that are attached both to them and to other (attached) sessions.
  */
 
 void
@@ -49,7 +48,7 @@ recalculate_sizes(void)
 	struct session	*s;
 	struct client	*c;
 	struct window	*w;
-	u_int		 i, j, ssx, ssy;
+	u_int		 i, j, ssx, ssy, slines, has;
 
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
@@ -74,9 +73,10 @@ recalculate_sizes(void)
 		}
 		s->flags &= ~SESSION_UNATTACHED;
 
-		if (ssy < status_lines)
-			ssy = status_lines + 1;
-		ssy -= status_lines;
+		slines = options_get_number(&s->options, "status-lines");
+		if (ssy < slines)
+			ssy = slines + 1;
+		ssy -= slines;
 		if (s->sx == ssx && s->sy == ssy)
 			continue;
 
@@ -97,7 +97,11 @@ recalculate_sizes(void)
 			s = ARRAY_ITEM(&sessions, j);
 			if (s == NULL || s->flags & SESSION_UNATTACHED)
 				continue;
-			if (session_has(s, w)) {
+			if (w->flags & WINDOW_AGGRESSIVE)
+				has = s->curw->window == w;
+			else
+				has = session_has(s, w);
+			if (has) {
 				if (s->sx < ssx)
 					ssx = s->sx;
 				if (s->sy < ssy)
@@ -110,6 +114,11 @@ recalculate_sizes(void)
 		}
 		w->flags &= ~WINDOW_HIDDEN;
 
+		if (ssx > w->limitx)
+			ssx = w->limitx;
+		if (ssy > w->limity)
+			ssy = w->limity;
+
 		if (screen_size_x(&w->base) == ssx &&
 		    screen_size_y(&w->base) == ssy)
 			continue;
@@ -117,7 +126,6 @@ recalculate_sizes(void)
 		log_debug("window size %u,%u (was %u,%u)", ssx, ssy,
 		    screen_size_x(&w->base), screen_size_y(&w->base));
 
-		server_clear_window(w);
 		window_resize(w, ssx, ssy);
 		server_redraw_window(w);
 	}
