@@ -1,4 +1,4 @@
-/* $Id: screen.c,v 1.59 2008/01/03 21:32:11 nicm Exp $ */
+/* $Id: screen.c,v 1.61 2008/06/14 12:05:06 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -128,7 +128,7 @@ screen_stringcolour(const char *s)
 
 /* Create a new screen. */
 void
-screen_create(struct screen *s, u_int dx, u_int dy)
+screen_create(struct screen *s, u_int dx, u_int dy, u_int hlimit)
 {
 	s->dx = dx;
 	s->dy = dy;
@@ -139,7 +139,7 @@ screen_create(struct screen *s, u_int dx, u_int dy)
 	s->rlower = s->dy - 1;
 
 	s->hsize = 0;
-	s->hlimit = history_limit;
+	s->hlimit = hlimit;
 
 	s->attr = SCREEN_DEFATTR;
 	s->colr = SCREEN_DEFCOLR;
@@ -297,13 +297,8 @@ void
 screen_set_cell(struct screen *s,
     u_int cx, u_int cy, u_char data, u_char attr, u_char colr)
 {
-	if (cx >= s->grid_size[cy]) {
-		if (data == SCREEN_DEFDATA &&
-		    attr == SCREEN_DEFATTR &&
-		    colr == SCREEN_DEFCOLR)
-			return;
+	if (cx >= s->grid_size[cy])
 		screen_expand_line(s, cy, cx + 1);
-	}
 
 	s->grid_data[cy][cx] = data;
 	s->grid_attr[cy][cx] = attr;
