@@ -1,4 +1,4 @@
-/* $Id: server-fn.c,v 1.36 2007/12/06 09:46:23 nicm Exp $ */
+/* $Id: server-fn.c,v 1.42 2008/06/16 17:35:40 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -23,57 +23,6 @@
 
 #include "tmux.h"
 
-/* Find session from command message. */
-struct session *
-server_extract_session(struct msg_command_data *data, char *name, char **cause)
-{
-	struct session *s;
-	u_int		i, n;
-
-	if (name != NULL) {
-		if ((s = session_find(name)) == NULL) {
-			xasprintf(cause, "session not found: %s", name);
-			return (NULL);
-		}
-		return (s);
-	}
-
-	if (data->pid != -1) {
-		if (data->pid != getpid()) {
-			xasprintf(cause, "wrong server: %lld", data->pid);
-			return (NULL);
-		}
-		if (data->idx > ARRAY_LENGTH(&sessions)) {
-			xasprintf(cause, "index out of range: %d", data->idx);
-			return (NULL);
-		}
-		if ((s = ARRAY_ITEM(&sessions, data->idx)) == NULL) {
-			xasprintf(
-			    cause, "session doesn't exist: %u", data->idx);
-			return (NULL);
-		}
-		return (s);
-	}
-
-	s = NULL;
-	n = 0;
-	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
-		if (ARRAY_ITEM(&sessions, i) != NULL) {
-			s = ARRAY_ITEM(&sessions, i);
-			n++;
-		}
-	}
-	if (s == NULL) {
-		xasprintf(cause, "no sessions found");
-		return (NULL);
-	}
-	if (n != 1) {
-		xasprintf(cause, "multiple sessions and session not specified");
-		return (NULL);
-	}
-	return (s);
-}
-
 void
 server_write_client(
     struct client *c, enum hdrtype type, const void *buf, size_t len)
@@ -99,7 +48,9 @@ server_write_session(
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
-		if (c != NULL && c->session == s)
+		if (c == NULL || c->session == NULL)
+			continue;
+		if (c->session == s)
 			server_write_client(c, type, buf, len);
 	}
 }
@@ -120,91 +71,61 @@ server_write_window(
 	}
 }
 
-void
-server_clear_client(struct client *c)
-{
-	struct screen_redraw_ctx	ctx;
-
-	screen_redraw_start_client(&ctx, c);
-	screen_redraw_set_attributes(&ctx, SCREEN_DEFATTR, SCREEN_DEFCOLR);
-	screen_redraw_clear_screen(&ctx);
-	screen_redraw_stop(&ctx);
-
-	status_write_client(c);
-}
-
 void
 server_redraw_client(struct client *c)
 {
-	struct screen_redraw_ctx	ctx;
-
-	screen_redraw_start_client(&ctx, c);
-	screen_redraw_lines(&ctx, 0, screen_size_y(ctx.s));
-	screen_redraw_stop(&ctx);
-
-	status_write_client(c);
+	c->flags |= CLIENT_REDRAW;
 }
 
 void
 server_status_client(struct client *c)
 {
-	status_write_client(c);
-}
-
-void
-server_clear_session(struct session *s)
-{
-	struct screen_redraw_ctx	ctx;
-
-	screen_redraw_start_session(&ctx, s);
-	screen_redraw_set_attributes(&ctx, SCREEN_DEFATTR, SCREEN_DEFCOLR);
-	screen_redraw_clear_screen(&ctx);
-	screen_redraw_stop(&ctx);
-
-	status_write_session(s);
+	c->flags |= CLIENT_STATUS;
 }
 
 void
 server_redraw_session(struct session *s)
 {
-	struct screen_redraw_ctx	ctx;
+	struct client	*c;
+	u_int		 i;
 
-	screen_redraw_start_session(&ctx, s);
-	screen_redraw_lines(&ctx, 0, screen_size_y(ctx.s));
-	screen_redraw_stop(&ctx);
-
-	status_write_session(s);
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL || c->session == NULL)
+			continue;
+		if (c->session == s)
+			server_redraw_client(c);
+	}
 }
 
 void
 server_status_session(struct session *s)
 {
-	status_write_session(s);
-}
+	struct client	*c;
+	u_int		 i;
 
-void
-server_clear_window(struct window *w)
-{
-	struct screen_redraw_ctx	ctx;
-
-	screen_redraw_start_window(&ctx, w);
-	screen_redraw_set_attributes(&ctx, SCREEN_DEFATTR, SCREEN_DEFCOLR);
-	screen_redraw_clear_screen(&ctx);
-	screen_redraw_stop(&ctx);
-
-	status_write_window(w);
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL || c->session == NULL)
+			continue;
+		if (c->session == s)
+			server_status_client(c);
+	}
 }
 
 void
 server_redraw_window(struct window *w)
 {
-	struct screen_redraw_ctx	ctx;
+	struct client	*c;
+	u_int		 i;
 
-	screen_redraw_start_window(&ctx, w);
-	screen_redraw_lines(&ctx, 0, screen_size_y(ctx.s));
-	screen_redraw_stop(&ctx);
-
-	status_write_window(w);
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL || c->session == NULL)
+			continue;
+		if (c->session->curw->window == w)
+			server_redraw_client(c);
+	}
 }
 
 void
@@ -233,32 +154,43 @@ server_write_message(struct client *c, const char *fmt, ...)
 	va_list				ap;
 	char			       *msg;
 	size_t				size;
+	u_int				slines;
+
+	slines = options_get_number(&c->session->options, "status-lines");
 
 	screen_redraw_start_client(&ctx, c);
-	screen_redraw_move_cursor(&ctx, 0, c->sy - 1);
-	screen_redraw_set_attributes(&ctx, ATTR_REVERSE, 0x88);
 
 	va_start(ap, fmt);
 	xvasprintf(&msg, fmt, ap);
 	va_end(ap);
 
+	msg = xrealloc(msg, 1, c->sx + 1);
+	msg[c->sx] = '\0';
+
 	size = strlen(msg);
-	if (size < c->sx - 1) {
-		msg = xrealloc(msg, 1, c->sx);
-		msg[c->sx - 1] = '\0';
-		memset(msg + size, SCREEN_DEFDATA, (c->sx - 1) - size);
-	}
+	if (size < c->sx)
+		memset(msg + size, ' ', c->sx - size);
+
+	screen_redraw_move_cursor(&ctx, 0, c->sy - 1);
+	screen_redraw_set_attributes(&ctx, ATTR_REVERSE, 0x88);
 	screen_redraw_write_string(&ctx, "%s", msg);
-	xfree(msg);
 
 	buffer_flush(c->tty.fd, c->tty.in, c->tty.out);
 	usleep(750000);
 
-	if (status_lines == 0) {
+	memset(msg, ' ', c->sx);
+
+	screen_redraw_move_cursor(&ctx, 0, c->sy - 1);
+	screen_redraw_set_attributes(&ctx, 0, 0x88);
+	screen_redraw_write_string(&ctx, "%s", msg);
+
+	xfree(msg);
+
+	if (slines == 0) {
 		screen_redraw_lines(&ctx, c->sy - 1, 1);
 		screen_redraw_stop(&ctx);
 	} else {
 		screen_redraw_stop(&ctx);
-		status_write_client(c);
+		server_status_client(c);
 	}
 }
