@@ -1,4 +1,4 @@
-/* $Id: session.c,v 1.30 2007/12/06 09:46:23 nicm Exp $ */
+/* $Id: session.c,v 1.37 2008/06/07 07:27:28 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,40 +29,65 @@
 struct sessions	sessions;
 
 void
-session_cancelbell(struct session *s, struct winlink *wl)
+session_alert_cancel(struct session *s, struct winlink *wl)
 {
-	u_int	i;
+	struct session_alert	*sa, *sb;
 
-	for (i = 0; i < ARRAY_LENGTH(&s->bells); i++) {
-		if (ARRAY_ITEM(&s->bells, i) == wl) {
-			ARRAY_REMOVE(&s->bells, i);
-			break;
+	sa = TAILQ_FIRST(&s->alerts);
+	while (sa != NULL) {
+		sb = sa;
+		sa = TAILQ_NEXT(sa, entry);
+
+		if (wl == NULL || sb->wl == wl) {
+			TAILQ_REMOVE(&s->alerts, sb, entry);
+			xfree(sb);
 		}
 	}
 }
 
 void
-session_addbell(struct session *s, struct window *w)
+session_alert_add(struct session *s, struct window *w, int type)
 {
-	struct winlink	*wl;
+	struct session_alert	*sa;
+	struct winlink		*wl;
 
 	RB_FOREACH(wl, winlinks, &s->windows) {
 		if (wl == s->curw)
 			continue;
-		if (wl->window == w && !session_hasbell(s, wl))
-			ARRAY_ADD(&s->bells, wl);
+
+		if (wl->window == w &&
+		    !session_alert_has(s, wl, type)) {
+			sa = xmalloc(sizeof *sa);
+			sa->wl = wl;
+			sa->type = type;
+			TAILQ_INSERT_HEAD(&s->alerts, sa, entry);
+		}
 	}
 }
 
 int
-session_hasbell(struct session *s, struct winlink *wl)
+session_alert_has(struct session *s, struct winlink *wl, int type)
 {
-	u_int	i;
+	struct session_alert	*sa;
 
-	for (i = 0; i < ARRAY_LENGTH(&s->bells); i++) {
-		if (ARRAY_ITEM(&s->bells, i) == wl)
+	TAILQ_FOREACH(sa, &s->alerts, entry) {
+		if (sa->wl == wl && sa->type == type)
 			return (1);
 	}
+
+	return (0);
+}
+
+int
+session_alert_has_window(struct session *s, struct window *w, int type)
+{
+	struct session_alert	*sa;
+
+	TAILQ_FOREACH(sa, &s->alerts, entry) {
+		if (sa->wl->window == w && sa->type == type)
+			return (1);
+	}
+
 	return (0);
 }
 
@@ -90,11 +115,13 @@ session_create(const char *name, const char *cmd, u_int sx, u_int sy)
 	u_int		 i;
 
 	s = xmalloc(sizeof *s);
-	s->tim = time(NULL);
+	if (clock_gettime(CLOCK_REALTIME, &s->ts) != 0)
+		fatal("clock_gettime failed");
 	s->curw = s->lastw = NULL;
 	RB_INIT(&s->windows);
-	ARRAY_INIT(&s->bells);
-
+	TAILQ_INIT(&s->alerts);
+	options_init(&s->options, &global_options);
+	
 	s->sx = sx;
 	s->sy = sy;
 
@@ -117,6 +144,8 @@ session_create(const char *name, const char *cmd, u_int sx, u_int sy)
 	}
 	session_select(s, 0);
 
+	log_debug("session %s created", s->name);
+
 	return (s);
 }
 
@@ -126,12 +155,17 @@ session_destroy(struct session *s)
 {
 	u_int	i;
 
+	log_debug("session %s destroyed", s->name);
+
 	if (session_index(s, &i) != 0)
 		fatalx("session not found");
 	ARRAY_SET(&sessions, i, NULL);
 	while (!ARRAY_EMPTY(&sessions) && ARRAY_LAST(&sessions) == NULL)
 		ARRAY_TRUNC(&sessions, 1);
 
+	session_alert_cancel(s, NULL);
+	options_free(&s->options);
+
 	while (!RB_EMPTY(&s->windows))
 		winlink_remove(&s->windows, RB_ROOT(&s->windows));
 
@@ -157,14 +191,15 @@ session_new(struct session *s, const char *name, const char *cmd, int idx)
 	struct window	*w;
 	const char	*env[] = { NULL, "TERM=screen", NULL };
 	char		 buf[256];
-	u_int		 i;
+	u_int		 i, hlimit;
 
 	if (session_index(s, &i) != 0)
 		fatalx("session not found");
 	xsnprintf(buf, sizeof buf, "TMUX=%ld,%u", (long) getpid(), i);
 	env[0] = buf;
 
-	if ((w = window_create(name, cmd, env, s->sx, s->sy)) == NULL)
+	hlimit = options_get_number(&s->options, "history-limit");
+	if ((w = window_create(name, cmd, env, s->sx, s->sy, hlimit)) == NULL)
 		return (NULL);
 	return (session_attach(s, w, idx));
 }
@@ -185,7 +220,7 @@ session_detach(struct session *s, struct winlink *wl)
 	if (s->lastw == wl)
 		s->lastw = NULL;
 
-	session_cancelbell(s, wl);
+	session_alert_cancel(s, wl);
 	winlink_remove(&s->windows, wl);
 	if (RB_EMPTY(&s->windows)) {
 		session_destroy(s);
@@ -223,7 +258,7 @@ session_next(struct session *s)
 		return (1);
 	s->lastw = s->curw;
 	s->curw = wl;
-	session_cancelbell(s, wl);
+	session_alert_cancel(s, wl);
 	return (0);
 }
 
@@ -243,7 +278,7 @@ session_previous(struct session *s)
 		return (1);
 	s->lastw = s->curw;
 	s->curw = wl;
-	session_cancelbell(s, wl);
+	session_alert_cancel(s, wl);
 	return (0);
 }
 
@@ -260,7 +295,7 @@ session_select(struct session *s, int idx)
 		return (1);
 	s->lastw = s->curw;
 	s->curw = wl;
-	session_cancelbell(s, wl);
+	session_alert_cancel(s, wl);
 	return (0);
 }
 
@@ -278,6 +313,6 @@ session_last(struct session *s)
 
 	s->lastw = s->curw;
 	s->curw = wl;
-	session_cancelbell(s, wl);
+	session_alert_cancel(s, wl);
 	return (0);
 }
