@@ -1,4 +1,4 @@
-/* $Id: tmux.c,v 1.46 2007/12/06 18:28:55 nicm Exp $ */
+/* $Id: tmux.c,v 1.56 2008/06/16 17:35:40 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -42,36 +42,24 @@ const char	*_malloc_options = "AJX";
 
 volatile sig_atomic_t sigwinch;
 volatile sig_atomic_t sigterm;
-char		*default_command;
+
+char		*cfg_file;
+struct options	 global_options;
 char		*paste_buffer;
-int		 bell_action;
+
 int		 debug_level;
-int		 prefix_key = META;
-u_char		 status_colour;
-u_int		 history_limit;
-u_int		 status_lines;
+int		 be_quiet;
 
 void		 sighandler(int);
+__dead void	 usage(void);
 
-void
-usage(char **ptr, const char *fmt, ...)
+__dead void
+usage(void)
 {
-	char	*msg;
-	va_list	 ap;
-
-#define USAGE \
-	"usage: %s [-v] [-S socket-path] [-s session-name] [-c client-tty]"
-	if (fmt == NULL) {
-		xasprintf(ptr, USAGE " command [flags]", __progname);
-	} else {
-		va_start(ap, fmt);
-		xvasprintf(&msg, fmt, ap);
-		va_end(ap);
-
-		xasprintf(ptr, USAGE " %s", __progname, msg);
-		xfree(msg);
-	}
-#undef USAGE
+	fprintf(stderr, 
+	    "usage: %s [-v] [-f file] [-S socket-path] [command [flags]]\n",
+	    __progname);
+	exit(1);
 }
 
 void
@@ -182,21 +170,21 @@ main(int argc, char **argv)
 	struct hdr	 	 hdr;
 	const char		*shell;
 	struct passwd		*pw;
-	char			*client, *path, *name, *cause;
+	char			*client, *path, *name, *cause, *home;
 	char			 rpath[MAXPATHLEN];
 	int	 		 n, opt;
 
 	client = path = name = NULL;
-        while ((opt = getopt(argc, argv, "c:S:s:vV")) != EOF) {
+        while ((opt = getopt(argc, argv, "f:S:qvV")) != EOF) {
                 switch (opt) {
-		case 'c':
-			client = xstrdup(optarg);
+		case 'f':
+			cfg_file = xstrdup(optarg);
 			break;
 		case 'S':
 			path = xstrdup(optarg);
 			break;
-		case 's':
-			name = xstrdup(optarg);
+		case 'q':
+			be_quiet = 1;
 			break;
 		case 'v':
 			debug_level++;
@@ -205,26 +193,48 @@ main(int argc, char **argv)
 			printf("%s " BUILD "\n", __progname);
 			exit(0);
                 default:
-			goto usage;
+			usage();
                 }
         }
 	argc -= optind;
 	argv += optind;
-	if (argc == 0)
-		goto usage;
 
 	log_open(stderr, LOG_USER, debug_level);
 	siginit();
 
-	status_lines = 1;
-	status_colour = 0x02;
-
-	bell_action = BELL_ANY;
-
-	history_limit = 2000;
+	options_init(&global_options, NULL);
+	options_set_number(&global_options, "status-lines", 1);
+	options_set_colours(&global_options, "status-colour", 0x02);
+	options_set_number(&global_options, "bell-action", BELL_ANY);
+	options_set_number(&global_options, "history-limit", 2000);
+	options_set_key(&global_options, "prefix-key", META);
+	options_set_string(&global_options, "status-left", "");
+	options_set_string(
+	    &global_options, "status-right", "%%H:%%M %%d-%%b-%%y");
+	options_set_number(&global_options, "status-interval", 15);
 
 	paste_buffer = NULL;
 
+	if (cfg_file == NULL) {
+		home = getenv("HOME");
+		if (home == NULL || *home == '\0') {
+			pw = getpwuid(getuid());
+			if (pw != NULL)
+				home = pw->pw_dir;
+			endpwent();
+		}
+		xasprintf(&cfg_file, "%s/%s", home, DEFAULT_CFG);
+		if (access(cfg_file, R_OK) != 0) {
+			xfree(cfg_file);
+			cfg_file = NULL;
+		}
+	} else {
+		if (access(cfg_file, R_OK) != 0) {
+			log_warn("%s", cfg_file);
+			exit(1);
+		}		
+	}
+
 	if (path == NULL) {
 		xasprintf(&path,
 		    "%s/%s-%lu", _PATH_TMP, __progname, (u_long) getuid());
@@ -255,24 +265,23 @@ main(int argc, char **argv)
 		if (shell == NULL || *shell == '\0')
 			shell = _PATH_BSHELL;
 	}
-	xasprintf(&default_command, "exec %s", shell);
+	options_set_string(
+	    &global_options, "default-command", "exec %s", shell);
 
-	if ((cmd = cmd_parse(argc, argv, &cause)) == NULL) {
-		if (cause == NULL)
-			goto usage;
+	if (argc == 0) {
+		cmd = xmalloc(sizeof *cmd);
+		cmd->entry = &cmd_new_session_entry;
+		cmd->entry->init(cmd, 0);
+	} else if ((cmd = cmd_parse(argc, argv, &cause)) == NULL) {
 		log_warnx("%s", cause);
 		exit(1);
 	}
 
 	memset(&cctx, 0, sizeof cctx);
-	if (!(cmd->entry->flags & CMD_NOSESSION) ||
-	    (cmd->entry->flags & CMD_CANTNEST))
-		client_fill_session(&data);
+	client_fill_session(&data);
 	if (client_init(rpath, &cctx, cmd->entry->flags & CMD_STARTSERVER) != 0)
 		exit(1);
 	b = buffer_create(BUFSIZ);
-	cmd_send_string(b, name);
-	cmd_send_string(b, client);
 	cmd_send(cmd, b);
 	cmd_free(cmd);
 
@@ -280,9 +289,6 @@ main(int argc, char **argv)
 	    MSG_COMMAND, &data, sizeof data, BUFFER_OUT(b), BUFFER_USED(b));
 	buffer_destroy(b);
 
-	if (name != NULL)
-		xfree(name);
-
 	for (;;) {
 		pfd.fd = cctx.srv_fd;
 		pfd.events = POLLIN;
@@ -334,7 +340,7 @@ main(int argc, char **argv)
 	}
 
 out:
-	xfree(default_command);
+	options_free(&global_options);
 
 	close(cctx.srv_fd);
 	buffer_destroy(cctx.srv_in);
@@ -344,9 +350,4 @@ out:
 	xmalloc_report(getpid(), "client");
 #endif
 	return (n);
-
-usage:
-	usage(&cause, NULL);
-	fprintf(stderr, "%s\n", cause);
-	exit(1);
 }
