@@ -1,4 +1,4 @@
-/* $Id: tmux.h,v 1.114 2008/01/03 21:32:11 nicm Exp $ */
+/* $Id: tmux.h,v 1.144 2008/06/16 17:35:40 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -36,6 +36,7 @@
 #include "compat/tree.h"
 #endif
 
+#include <limits.h>
 #include <poll.h>
 #include <signal.h>
 #include <stdarg.h>
@@ -58,10 +59,20 @@ extern char    *__progname;
 #define __packed __attribute__ ((__packed__))
 #endif
 
+#ifndef timespeccmp
+#define timespeccmp(tsp, usp, cmp)                                      \
+        (((tsp)->tv_sec == (usp)->tv_sec) ?                             \
+            ((tsp)->tv_nsec cmp (usp)->tv_nsec) :                       \
+            ((tsp)->tv_sec cmp (usp)->tv_sec))
+#endif
+
 #ifndef TTY_NAME_MAX
 #define TTY_NAME_MAX 32
 #endif
 
+/* Default configuration file. */
+#define DEFAULT_CFG ".tmux.conf"
+
 /* Fatal errors. */
 #define fatal(msg) log_fatal("%s: %s", __func__, msg);
 #define fatalx(msg) log_fatalx("%s: %s", __func__, msg);
@@ -101,12 +112,17 @@ struct buffer {
 	size_t	 off;		/* offset of data in buffer */
 };
 
+/* Bell option values. */
+#define BELL_NONE 0
+#define BELL_ANY 1
+#define BELL_CURRENT 2
+
 /* Key codes. ncurses defines KEY_*. Grrr. */
 #define KEYC_NONE 256
 #define KEYC_A1 -1
 #define KEYC_A3 -2
 #define KEYC_B2 -3
-/* XXX #define KEYC_BACKSPACE -4 */
+/* #define KEYC_BACKSPACE -4 */
 #define KEYC_BEG -5
 #define KEYC_BTAB -6
 #define KEYC_C1 -7
@@ -509,6 +525,12 @@ struct window {
 	int		 flags;
 #define WINDOW_BELL 0x1
 #define WINDOW_HIDDEN 0x2
+#define WINDOW_ACTIVITY 0x4
+#define WINDOW_MONITOR 0x8
+#define WINDOW_AGGRESSIVE 0x10
+
+	u_int		 limitx;
+	u_int		 limity;
 
 	struct screen	*screen;
 	struct screen	 base;
@@ -529,10 +551,42 @@ struct winlink {
 };
 RB_HEAD(winlinks, winlink);
 
+/* Option data structures. */
+struct options_entry {
+	char		*name;
+
+	enum {
+		OPTIONS_STRING,
+		OPTIONS_NUMBER,
+		OPTIONS_KEY,
+		OPTIONS_COLOURS
+	} type;
+	union {
+		char	*string;
+		long long number;
+		int	 key;
+		u_char	 colours;
+	} value;
+
+	SPLAY_ENTRY(options_entry) entry;
+};
+
+struct options {
+	SPLAY_HEAD(options_tree, options_entry) tree;
+	struct options	*parent;
+};
+
 /* Client session. */
+struct session_alert {
+	struct winlink	*wl;
+	int		 type;
+
+	TAILQ_ENTRY(session_alert) entry;
+};
+
 struct session {
 	char		*name;
-	time_t		 tim;
+	struct timespec	 ts;
 
 	u_int		 sx;
 	u_int		 sy;
@@ -541,7 +595,9 @@ struct session {
 	struct winlink	*lastw;
 	struct winlinks	 windows;
 
-	ARRAY_DECL(, struct winlink *) bells;	/* windows with bells */
+	struct options	 options;
+
+	TAILQ_HEAD(, session_alert) alerts;
 
 #define SESSION_UNATTACHED 0x1	/* not attached to any clients */
 	int		 flags;
@@ -592,6 +648,7 @@ struct client {
 	struct buffer	*out;
 
 	struct tty 	 tty;
+	struct timespec	 status_ts;
 
 	u_int		 sx;
 	u_int		 sy;
@@ -599,6 +656,9 @@ struct client {
 #define CLIENT_TERMINAL 0x1
 #define CLIENT_PREFIX 0x2
 #define CLIENT_MOUSE 0x4
+#define CLIENT_REDRAW 0x8
+#define CLIENT_STATUS 0x10
+#define CLIENT_CLEAR 0x20
 	int		 flags;
 
 	struct session	*session;
@@ -620,37 +680,56 @@ struct client_ctx {
 struct cmd_ctx {
 	struct client  *cmdclient;
 
-	struct client  *client;
-	struct session *session;
+	struct client  *curclient;
+	struct session *cursession;
+	struct msg_command_data	*msgdata;
 
 	void		(*print)(struct cmd_ctx *, const char *, ...);
+	void		(*info)(struct cmd_ctx *, const char *, ...);
 	void		(*error)(struct cmd_ctx *, const char *, ...);
 
 #define CMD_KEY 0x1
 	int		flags;
 };
 
+struct cmd {
+	const struct cmd_entry *entry;
+	void	       	*data;
+};
+
 struct cmd_entry {
 	const char	*name;
 	const char	*alias;
 	const char	*usage;
 
 #define CMD_STARTSERVER 0x1
-#define CMD_NOSESSION 0x2
-#define CMD_NOCLIENT 0x4
-#define CMD_CANTNEST 0x8
+#define CMD_CANTNEST 0x2
+#define CMD_KFLAG 0x4
+#define CMD_DFLAG 0x8
+#define CMD_ONEARG 0x10
 	int		 flags;
 
-	int		 (*parse)(void **, int, char **, char **);
-	void		 (*exec)(void *, struct cmd_ctx *);
-	void		 (*send)(void *, struct buffer *);
-	void	         (*recv)(void **, struct buffer *);
-	void		 (*free)(void *);
+	void		 (*init)(struct cmd *, int);
+	int		 (*parse)(struct cmd *, int, char **, char **);
+	void		 (*exec)(struct cmd *, struct cmd_ctx *);
+	void		 (*send)(struct cmd *, struct buffer *);
+	void	         (*recv)(struct cmd *, struct buffer *);
+	void		 (*free)(struct cmd *);
+	void 		 (*print)(struct cmd *, char *, size_t);
 };
 
-struct cmd {
-	const struct cmd_entry *entry;
-	void	       	*data;
+/* Generic command data. */
+struct cmd_target_data {
+	int	 flags;
+	char	*target;
+	char	*arg;
+};
+
+struct cmd_srcdst_data {
+	int	 flags;
+	char	*src;
+	char	*dst;
+	char	*arg;
 };
 
 /* Key binding. */
@@ -678,22 +757,33 @@ size_t	 	 strlcat(char *, const char *, size_t);
 /* tmux.c */
 extern volatile sig_atomic_t sigwinch;
 extern volatile sig_atomic_t sigterm;
-#define BELL_NONE 0
-#define BELL_ANY 1
-#define BELL_CURRENT 2
-extern char	*default_command;
+extern struct options global_options;
+extern char	*cfg_file;
 extern char	*paste_buffer;
-extern int	 bell_action;
 extern int	 debug_level;
-extern int	 prefix_key;
-extern u_char	 status_colour;
-extern u_int	 history_limit;
-extern u_int	 status_lines;
-void		 usage(char **, const char *, ...);
+extern int	 be_quiet;
 void		 logfile(const char *);
 void		 siginit(void);
 void		 sigreset(void);
 
+/* cfg.c */
+int		 load_cfg(const char *, char **x);
+
+/* options.c */
+int	options_cmp(struct options_entry *, struct options_entry *);
+SPLAY_PROTOTYPE(options_tree, options_entry, entry, options_cmp);
+void	options_init(struct options *, struct options *);
+void	options_free(struct options *);
+void printflike3 options_set_string(
+    	    struct options *, const char *, const char *, ...);
+char   *options_get_string(struct options *, const char *);
+void	options_set_number(struct options *, const char *, long long);
+long long options_get_number(struct options *, const char *);
+void	options_set_key(struct options *, const char *, int);
+int	options_get_key(struct options *, const char *);
+void	options_set_colours(struct options *, const char *, u_char);
+u_char	options_get_colours(struct options *, const char *);
+
 /* tty.c */
 void		 tty_init(struct tty *, char *, char *);
 int		 tty_open(struct tty *, char **);
@@ -716,6 +806,11 @@ void		 tty_vwrite_window(void *, int, va_list);
 void		 tty_write_session(void *, int, ...);
 void		 tty_vwrite_session(void *, int, va_list);
 
+/* arg.c */
+struct client 	*arg_parse_client(const char *);
+struct session 	*arg_parse_session(const char *);
+int		 arg_parse_window(const char *, struct session **, int *);
+
 /* cmd.c */
 struct cmd	*cmd_parse(int, char **, char **);
 void		 cmd_exec(struct cmd *, struct cmd_ctx *);
@@ -724,11 +819,17 @@ struct cmd	*cmd_recv(struct buffer *);
 void		 cmd_free(struct cmd *);
 void		 cmd_send_string(struct buffer *, const char *);
 char		*cmd_recv_string(struct buffer *);
+struct session	*cmd_current_session(struct cmd_ctx *);
+struct client	*cmd_find_client(struct cmd_ctx *, const char *);
+struct session	*cmd_find_session(struct cmd_ctx *, const char *);
+struct winlink	*cmd_find_window(
+    		     struct cmd_ctx *, const char *, struct session **);
 extern const struct cmd_entry cmd_attach_session_entry;
 extern const struct cmd_entry cmd_bind_key_entry;
 extern const struct cmd_entry cmd_copy_mode_entry;
 extern const struct cmd_entry cmd_detach_client_entry;
 extern const struct cmd_entry cmd_has_session_entry;
+extern const struct cmd_entry cmd_kill_server_entry;
 extern const struct cmd_entry cmd_kill_session_entry;
 extern const struct cmd_entry cmd_kill_window_entry;
 extern const struct cmd_entry cmd_last_window_entry;
@@ -747,13 +848,39 @@ extern const struct cmd_entry cmd_rename_session_entry;
 extern const struct cmd_entry cmd_rename_window_entry;
 extern const struct cmd_entry cmd_scroll_mode_entry;
 extern const struct cmd_entry cmd_select_window_entry;
+extern const struct cmd_entry cmd_send_keys_entry;
 extern const struct cmd_entry cmd_send_prefix_entry;
 extern const struct cmd_entry cmd_set_option_entry;
+extern const struct cmd_entry cmd_set_window_option_entry;
+extern const struct cmd_entry cmd_show_options_entry;
+extern const struct cmd_entry cmd_show_window_options_entry;
+extern const struct cmd_entry cmd_start_server_entry;
 extern const struct cmd_entry cmd_swap_window_entry;
 extern const struct cmd_entry cmd_switch_client_entry;
 extern const struct cmd_entry cmd_unbind_key_entry;
 extern const struct cmd_entry cmd_unlink_window_entry;
-void	cmd_select_window_default(void **, int);
+
+/* cmd-generic.c */
+#define CMD_TARGET_WINDOW_USAGE "[-t target-window]"
+#define CMD_TARGET_SESSION_USAGE "[-t target-session]"
+#define CMD_TARGET_CLIENT_USAGE "[-t target-client]"
+void	cmd_target_init(struct cmd *, int);
+int	cmd_target_parse(struct cmd *, int, char **, char **);
+void	cmd_target_exec(struct cmd *, struct cmd_ctx *);
+void	cmd_target_send(struct cmd *, struct buffer *);
+void	cmd_target_recv(struct cmd *, struct buffer *);
+void	cmd_target_free(struct cmd *);
+void	cmd_target_print(struct cmd *, char *, size_t);
+#define CMD_SRCDST_WINDOW_USAGE "[-s src-window] [-t dst-window]"
+#define CMD_SRCDST_SESSION_USAGE "[-s src-session] [-t dst-session]"
+#define CMD_SRCDST_CLIENT_USAGE "[-s src-client] [-t dst-client]"
+void	cmd_srcdst_init(struct cmd *, int);
+int	cmd_srcdst_parse(struct cmd *, int, char **, char **);
+void	cmd_srcdst_exec(struct cmd *, struct cmd_ctx *);
+void	cmd_srcdst_send(struct cmd *, struct buffer *);
+void	cmd_srcdst_recv(struct cmd *, struct buffer *);
+void	cmd_srcdst_free(struct cmd *);
+void	cmd_srcdst_print(struct cmd *, char *, size_t);
 
 /* client.c */
 int	 client_init(const char *, struct client_ctx *, int);
@@ -797,21 +924,16 @@ void	 server_write_session(
              struct session *, enum hdrtype, const void *, size_t);
 void	 server_write_window(
              struct window *, enum hdrtype, const void *, size_t);
-void	 server_clear_client(struct client *);
 void	 server_redraw_client(struct client *);
 void	 server_status_client(struct client *);
-void	 server_clear_session(struct session *);
 void	 server_redraw_session(struct session *);
 void	 server_status_session(struct session *);
-void	 server_clear_window(struct window *);
 void	 server_redraw_window(struct window *);
 void	 server_status_window(struct window *);
 void printflike2 server_write_message(struct client *, const char *, ...);
 
 /* status.c */
-void	 status_write_client(struct client *);
-void	 status_write_session(struct session *);
-void	 status_write_window(struct window *);
+void	 status_redraw(struct client *c);
 
 /* resize.c */
 void	 recalculate_sizes(void);
@@ -892,16 +1014,16 @@ void	screen_redraw_move_cursor(struct screen_redraw_ctx *, u_int, u_int);
 void	screen_redraw_set_attributes(struct screen_redraw_ctx *, u_int, u_int);
 void printflike2 screen_redraw_write_string(
     	     struct screen_redraw_ctx *, const char *, ...);
-void	screen_redraw_clear_screen(struct screen_redraw_ctx *);
 void	screen_redraw_cell(struct screen_redraw_ctx *, u_int, u_int);
 void	screen_redraw_area(
     	    struct screen_redraw_ctx *, u_int, u_int, u_int, u_int);
 void	screen_redraw_lines(struct screen_redraw_ctx *, u_int, u_int);
+void	screen_redraw_columns(struct screen_redraw_ctx *, u_int, u_int);
 
 /* screen.c */
 const char *screen_colourstring(u_char);
 u_char	 screen_stringcolour(const char *);
-void	 screen_create(struct screen *, u_int, u_int);
+void	 screen_create(struct screen *, u_int, u_int, u_int);
 void	 screen_destroy(struct screen *);
 void	 screen_resize(struct screen *, u_int, u_int);
 void	 screen_expand_line(struct screen *, u_int, u_int);
@@ -931,8 +1053,8 @@ struct winlink	*winlink_add(struct winlinks *, struct window *, int);
 void		 winlink_remove(struct winlinks *, struct winlink *);
 struct winlink	*winlink_next(struct winlinks *, struct winlink *);
 struct winlink	*winlink_previous(struct winlinks *, struct winlink *);
-struct window	*window_create(
-    		     const char *, const char *, const char **, u_int, u_int);
+struct window	*window_create(const char *,
+		     const char *, const char **, u_int, u_int, u_int);
 void		 window_destroy(struct window *);
 int		 window_resize(struct window *, u_int, u_int);
 int		 window_set_mode(struct window *, const struct window_mode *);
@@ -953,13 +1075,14 @@ void printflike2 window_more_add(struct window *, const char *, ...);
 
 /* session.c */
 extern struct sessions sessions;
-void		 session_cancelbell(struct session *, struct winlink *);
-void		 session_addbell(struct session *, struct window *);
-int		 session_hasbell(struct session *, struct winlink *);
+void	 session_alert_add(struct session *, struct window *, int);
+void	 session_alert_cancel(struct session *, struct winlink *);
+int	 session_alert_has(struct session *, struct winlink *, int);
+int	 session_alert_has_window(struct session *, struct window *, int);
 struct session	*session_find(const char *);
 struct session	*session_create(const char *, const char *, u_int, u_int);
-void		 session_destroy(struct session *);
-int		 session_index(struct session *, u_int *);
+void	 	 session_destroy(struct session *);
+int	 	 session_index(struct session *, u_int *);
 struct winlink	*session_new(struct session *, const char *, const char *, int);
 struct winlink	*session_attach(struct session *, struct window *, int);
 int		 session_detach(struct session *, struct winlink *);
