@@ -1,4 +1,4 @@
-/* $Id: window-copy.c,v 1.15 2007/12/07 09:49:18 nicm Exp $ */
+/* $Id: window-copy.c,v 1.17 2008/06/04 18:50:35 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -91,7 +91,7 @@ window_copy_init(struct window *w)
 	data->cy = w->base.cy;
 
 	s = &data->screen;
-	screen_create(s, screen_size_x(&w->base), screen_size_y(&w->base));
+	screen_create(s, screen_size_x(&w->base), screen_size_y(&w->base), 0);
 	s->cx = data->cx;
 	s->cy = data->cy;
 
@@ -184,9 +184,11 @@ window_copy_key(struct window *w, int key)
 		window_copy_copy_selection(w);
 		window_reset_mode(w);
 		break;
+	case '0':
 	case '\001':	/* C-a */
 		window_copy_cursor_start_of_line(w);
 		break;
+	case '$':
 	case '\005':	/* C-e */
 		window_copy_cursor_end_of_line(w);
 		break;
@@ -202,7 +204,8 @@ window_copy_write_line(
 	size_t	 			 size;
 
 	if (py == 0) {
-		screen_write_set_attributes(ctx, 0, status_colour);
+		screen_write_set_attributes(
+		    ctx, ATTR_BRIGHT|ATTR_REVERSE, 0x70);
 		screen_write_move_cursor(ctx, 0, 0);
 		size = screen_write_put_string_rjust(
 		    ctx, "[%u,%u/%u]", data->ox, data->oy, w->base.hsize);
