@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.35 2008/01/02 19:22:21 nicm Exp $ */
+/* $Id: window.c,v 1.41 2008/06/14 16:47:20 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -139,8 +139,11 @@ winlink_next(unused struct winlinks *wwl, struct winlink *wl)
 }
 
 struct winlink *
-winlink_previous(struct winlinks *wwl, struct winlink *wl)
+winlink_previous(unused struct winlinks *wwl, struct winlink *wl)
 {
+#ifdef RB_PREV
+	return (RB_PREV(winlinks, wwl, wl));
+#else
 	struct winlink	*wk;
 	int		 idx = wl->idx;
 
@@ -153,11 +156,12 @@ winlink_previous(struct winlinks *wwl, struct winlink *wl)
 	if (wl == NULL)
 		return (NULL);
 	return (wk);
+#endif
 }
 
 struct window *
-window_create(
-    const char *name, const char *cmd, const char **env, u_int sx, u_int sy)
+window_create(const char *name, 
+    const char *cmd, const char **env, u_int sx, u_int sy, u_int hlimit)
 {
 	struct window	*w;
 	struct winsize	 ws;
@@ -169,7 +173,7 @@ window_create(
 	ws.ws_col = sx;
 	ws.ws_row = sy;
 
-	switch (forkpty(&fd, NULL, NULL, &ws)) {
+ 	switch (forkpty(&fd, NULL, NULL, &ws)) {
 	case -1:
 		return (NULL);
 	case 0:
@@ -178,8 +182,9 @@ window_create(
 				fatal("putenv failed");
 		}
 		sigreset();
+		log_debug("started child: cmd=%s; pid=%d", cmd, getpid());
 		log_close();
-
+		
 		execl(_PATH_BSHELL, "sh", "-c", cmd, (char *) NULL);
 		fatal("execl failed");
 	}
@@ -196,7 +201,9 @@ window_create(
 	w->in = buffer_create(BUFSIZ);
 	w->out = buffer_create(BUFSIZ);
 	w->mode = NULL;
-	screen_create(&w->base, sx, sy);
+	w->flags = 0;
+	w->limitx = w->limity = UINT_MAX;
+	screen_create(&w->base, sx, sy, hlimit);
 	w->screen = &w->base;
 	input_init(w);
 
